@@ -3,15 +3,21 @@ import contextlib
 import dataclasses
 import datetime
 import functools
+import io
 import mimetypes
 import os
 import os.path
+import shutil
 import tempfile
+import typing
+import zipfile
 from collections.abc import Callable, Generator
 from pathlib import Path
-from typing import Literal
+from typing import Any, Literal
 
 import boto3
+import fsspec
+import fsspec.utils
 from cloudpathlib import CloudPath, S3Client, S3Path
 from iker.common.utils.shutils import glob_match, listfile, path_depth
 from iker.common.utils.strutils import is_empty, trim_to_none
@@ -31,23 +37,35 @@ __all__ = [
     "S3TransferCallbackClient",
     "s3_make_progress_callback",
     "s3_make_progressed_client",
+    "s3_estimate_archive_ranged_requests",
+    "s3_open_archive_member",
+    "s3_open_archive_members",
 ]
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True, eq=True)
 class S3ObjectMeta(object):
     key: str
     last_modified: datetime.datetime
     size: int
 
 
+if typing.TYPE_CHECKING:
+    def s3_make_client(
+        access_key_id: str = None,
+        secret_access_key: str = None,
+        region_name: str = None,
+        endpoint_url: str = None,
+    ) -> contextlib.AbstractContextManager[S3Client]: ...
+
+
 @contextlib.contextmanager
 def s3_make_client(
     access_key_id: str = None,
     secret_access_key: str = None,
     region_name: str = None,
     endpoint_url: str = None,
-) -> Generator[S3Client]:
+) -> Generator[S3Client, None, None]:
     """
     Creates an S3 client as a context manager for safe resource handling.
 
@@ -60,7 +78,12 @@ def s3_make_client(
     session = boto3.Session(aws_access_key_id=trim_to_none(access_key_id),
                             aws_secret_access_key=trim_to_none(secret_access_key),
                             region_name=trim_to_none(region_name))
-    yield S3Client(boto3_session=session, endpoint_url=trim_to_none(endpoint_url))
+    client = S3Client(boto3_session=session, endpoint_url=trim_to_none(endpoint_url))
+    try:
+        yield client
+    finally:
+        if hasattr(client, "close"):
+            client.close()
 
 
 def s3_list_objects(client: S3Client, bucket: str, prefix: str, limit: int = None) -> Generator[S3ObjectMeta]:
@@ -386,6 +409,15 @@ def s3_make_progress_callback(
     return progress_callback
 
 
+if typing.TYPE_CHECKING:
+    def s3_make_progressed_client(
+        access_key_id: str = None,
+        secret_access_key: str = None,
+        region_name: str = None,
+        endpoint_url: str = None,
+    ) -> contextlib.AbstractContextManager[S3Client]: ...
+
+
 @contextlib.contextmanager
 def s3_make_progressed_client(
     access_key_id: str = None,
@@ -414,3 +446,211 @@ def s3_make_progressed_client(
         yield S3TransferCallbackClient(boto3_session=session,
                                        endpoint_url=trim_to_none(endpoint_url),
                                        transfer_callback=s3_make_progress_callback(progress))
+
+
+def s3_options_from_s3_client(client: S3Client) -> dict[str, Any]:
+    """
+    Extracts S3 connection options from an existing S3Client instance for use with ``fsspec``.
+
+    :param client: An instance of ``S3Client``.
+    :return: A dictionary of S3 connection options.
+    """
+    if client.sess is None:
+        return {}
+
+    s3_options: dict[str, Any] = {}
+
+    credentials = client.sess.get_credentials()
+    if credentials is not None:
+        if credentials.access_key:
+            s3_options["key"] = credentials.access_key
+        if credentials.secret_key:
+            s3_options["secret"] = credentials.secret_key
+        if credentials.token:
+            s3_options["token"] = credentials.token
+
+    client_kwargs = {}
+    if client.sess.region_name or client.client.meta.region_name:
+        client_kwargs["region_name"] = client.sess.region_name or client.client.meta.region_name
+    if client.client.meta.endpoint_url:
+        client_kwargs["endpoint_url"] = client.client.meta.endpoint_url
+
+    if client_kwargs:
+        s3_options["client_kwargs"] = client_kwargs
+
+    return s3_options
+
+
+def s3_estimate_archive_ranged_requests(
+    client: S3Client,
+    archive_url: str,
+    members: list[str],
+    *,
+    central_directory_overhead: int = 64 * 1024,
+    member_header_overhead: int = 128,
+) -> tuple[int, int, list[str]]:
+    """
+    Estimates the total size of ranged requests needed to extract specific members from a ZIP archive stored in S3.
+
+    :param client: An instance of ``S3Client``.
+    :param archive_url: The URL of the ZIP archive in S3.
+    :param members: List of member file names to extract from the archive.
+    :param central_directory_overhead: Estimated overhead size for the central directory.
+    :param member_header_overhead: Estimated overhead size for each member's header.
+
+    :return: A tuple containing:
+             - Estimated total size of ranged requests (in bytes).
+             - Total size of the archive (in bytes).
+             - List of member names that were not found in the archive.
+    """
+    s3_options = s3_options_from_s3_client(client)
+
+    storage_opts = fsspec.utils.infer_storage_options(archive_url)
+    protocol = storage_opts.get("protocol")
+    if protocol != "s3":
+        raise ValueError(f"unsupported protocol '{protocol}', only 's3' is supported")
+
+    fs = fsspec.filesystem("s3", **s3_options)
+
+    archive_size = fs.size(archive_url)
+
+    ranged_total_size = 0
+    missing_members = []
+
+    with fsspec.open(archive_url, "rb", s3=s3_options) as fh, zipfile.ZipFile(fh) as archive:
+        # central directory read happened here (small)
+        for member in members:
+            try:
+                member_info = archive.getinfo(member)
+                ranged_total_size += member_info.compress_size + member_header_overhead
+            except KeyError:
+                missing_members.append(member)
+                continue
+
+    ranged_total_size += central_directory_overhead
+
+    return ranged_total_size, archive_size, missing_members
+
+
+if typing.TYPE_CHECKING:
+    def s3_open_archive_member(
+        client: S3Client,
+        archive_url: str,
+        member: str,
+        mode: Literal["r", "rb"] = "r",
+    ) -> contextlib.AbstractContextManager[typing.IO]: ...
+
+
+@contextlib.contextmanager
+def s3_open_archive_member(
+    client: S3Client,
+    archive_url: str,
+    member: str,
+    mode: Literal["r", "rb"] = "r",
+) -> Generator[typing.IO, None, None]:
+    """
+    Opens a specific member file from a ZIP archive stored in S3.
+
+    :param client: An instance of ``S3Client``.
+    :param archive_url: The URL of the ZIP archive in S3.
+    :param member: The member file name to open from the archive.
+    :param mode: File mode for opening the member ("r" for text, "rb" for binary).
+
+    :return: A file-like object for the specified member within the ZIP archive.
+    """
+    s3_options = s3_options_from_s3_client(client)
+
+    storage_opts = fsspec.utils.infer_storage_options(archive_url)
+    protocol = storage_opts.get("protocol")
+    if protocol != "s3":
+        raise ValueError(f"unsupported protocol '{protocol}', only 's3' is supported")
+
+    with fsspec.open(f"zip://{member}::{archive_url}", mode, s3=s3_options) as fh:
+        yield fh
+
+
+def s3_open_archive_members(
+    client: S3Client,
+    archive_url: str,
+    members: list[str],
+    mode: Literal["r", "rb"] = "r",
+    *,
+    threshold: float = 0.5,
+    central_directory_overhead: int = 64 * 1024,
+    member_header_overhead: int = 128,
+) -> Generator[tuple[str, Callable[[], typing.IO]], None, None]:
+    """
+    Choose the best transfer strategy (ranged requests per-member vs full archive transfer)
+    based on estimated transfer size ratio and yield callables to open each requested member.
+
+    The callables return file-like objects for each member when invoked. Due to lazy evaluation,
+    the actual data transfer occurs when the member is opened by the corresponding callable.
+    Thus, the callables must be used immediately after being yielded, to avoid issues with temporary
+    file lifetimes.
+
+    Example usage:
+
+    >>> for member, opener in s3_open_archive_members(client, archive_url, members):
+    ...     with opener() as fh:
+    ...         data = fh.read()
+
+    Incorrect usage that may lead to errors due to temporary file cleanup:
+
+    >>> openers = []
+    >>> for member, opener in s3_open_archive_members(client, archive_url, members):
+    ...     openers.append((member, opener))
+    >>> for member, opener in openers:
+    ...     with opener() as fh:  # May fail if temporary files have been cleaned up
+    ...         data = fh.read()
+
+    :param client: An instance of ``S3Client``.
+    :param archive_url: S3 URL to the ZIP archive.
+    :param members: List of member names to stream.
+    :param mode: File mode for opening members ("r" for text, "rb" for binary).
+    :param threshold: If (estimated ranged transfer bytes / archive bytes) <= threshold,
+                      use ranged per-member access; otherwise download the whole archive.
+    :param central_directory_overhead: Passed to s3_estimate_archive_ranged_requests.
+    :param member_header_overhead: Passed to s3_estimate_archive_ranged_requests.
+
+    :return: An iterable of callables that return file-like objects for each requested member.
+    """
+    s3_options = s3_options_from_s3_client(client)
+
+    ranged_total, archive_size, missing = s3_estimate_archive_ranged_requests(
+        client,
+        archive_url,
+        members,
+        central_directory_overhead=central_directory_overhead,
+        member_header_overhead=member_header_overhead,
+    )
+
+    if len(missing) > 0:
+        raise FileNotFoundError(f"archive missing members: {missing}")
+
+    # Avoid division by zero; prefer ranged if archive size is zero (degenerate case)
+    use_ranged = True
+    if archive_size > 0:
+        use_ranged = (ranged_total / archive_size) <= threshold
+
+    if use_ranged:
+        # Use fsspec zip:// style per-member access (this will issue ranged requests)
+        for member in members:
+            opener = functools.partial(s3_open_archive_member, client, archive_url, member, mode)
+            yield member, opener
+        return
+
+    # Download full archive once and serve members from it (read member bytes into memory)
+    with fsspec.open(archive_url, "rb", s3=s3_options) as fh, tempfile.TemporaryFile() as temp_fh:
+        shutil.copyfileobj(fh, temp_fh)
+        temp_fh.seek(0)
+        with zipfile.ZipFile(temp_fh) as archive:
+            for member in members:
+                try:
+                    if mode == "r":
+                        opener = lambda m=member: io.TextIOWrapper(archive.open(m), encoding="utf-8")
+                    else:
+                        opener = lambda m=member: io.BytesIO(archive.read(m))
+                    yield member, opener
+                except KeyError as e:
+                    # Shouldn't happen due to earlier check, but guard anyway
+                    raise FileNotFoundError(member) from e
