@@ -1,21 +1,36 @@
+import contextlib
+import os
 import tempfile
 import unittest
 
 import ddt
 import moto
+import moto.server
 from iker.common.utils.randutils import randomizer
 from iker.common.utils.shutils import listfile
 from iker.common.utils.testutils import norm_path
 
 from plexus.common.utils.s3utils import *
+from plexus.common.utils.s3utils import s3_estimate_archive_ranged_requests
 from plexus_tests import resources_directory
 
 
+@contextlib.contextmanager
+def moto_server_endpoint_url():
+    server = moto.server.ThreadedMotoServer(port=0)  # random free port
+    try:
+        server.start()
+        host, port = server.get_host_and_port()
+        yield f"http://{host}:{port}"
+    finally:
+        server.stop()
+
+
 @ddt.ddt
 class S3UtilsTest(unittest.TestCase):
 
     def test_s3_list_object__random_text_files(self):
-        with moto.mock_aws(), s3_make_client(region_name="us-east-1") as client:
+        with moto.mock_aws(), s3_make_client() as client:
             client.client.create_bucket(Bucket="dummy-bucket")
 
             data = []
@@ -33,7 +48,7 @@ class S3UtilsTest(unittest.TestCase):
                 self.assertTrue(any(o.key == key for o in result))
                 self.assertEqual(s3_pull_text(client, "dummy-bucket", key), text)
 
-        with moto.mock_aws(), s3_make_progressed_client(region_name="us-east-1") as client:
+        with moto.mock_aws(), s3_make_progressed_client() as client:
             client.client.create_bucket(Bucket="dummy-bucket")
 
             data = []
@@ -52,7 +67,7 @@ class S3UtilsTest(unittest.TestCase):
                 self.assertEqual(client.CloudPath("s3://dummy-bucket", key).read_text(), text)
 
     def test_s3_listfile__random_text_files(self):
-        with moto.mock_aws(), s3_make_client(region_name="us-east-1") as client:
+        with moto.mock_aws(), s3_make_client() as client:
             client.client.create_bucket(Bucket="dummy-bucket")
 
             data = []
@@ -70,7 +85,7 @@ class S3UtilsTest(unittest.TestCase):
                 self.assertTrue(any(o.key == key for o in result))
                 self.assertEqual(s3_pull_text(client, "dummy-bucket", key), text)
 
-        with moto.mock_aws(), s3_make_progressed_client(region_name="us-east-1") as client:
+        with moto.mock_aws(), s3_make_progressed_client() as client:
             client.client.create_bucket(Bucket="dummy-bucket")
 
             data = []
@@ -90,7 +105,7 @@ class S3UtilsTest(unittest.TestCase):
 
     data_s3_listfile = [
         (
-            "{res_dir}/unittest/s3utils/",
+            "unittest/s3utils/",
             "dummy-bucket",
             "unittest/s3utils/",
             [],
@@ -110,7 +125,7 @@ class S3UtilsTest(unittest.TestCase):
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils/",
             [],
@@ -130,7 +145,7 @@ class S3UtilsTest(unittest.TestCase):
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils/dir.foo",
+            "unittest/s3utils/dir.foo",
             "dummy-bucket",
             "unittest/s3utils/dir.foo",
             [],
@@ -147,7 +162,7 @@ class S3UtilsTest(unittest.TestCase):
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils/dir.baz",
+            "unittest/s3utils/dir.baz",
             "dummy-bucket",
             "unittest/s3utils/dir.baz",
             [],
@@ -160,7 +175,7 @@ class S3UtilsTest(unittest.TestCase):
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils/",
             ["*.foo", "*.bar"],
@@ -172,7 +187,7 @@ class S3UtilsTest(unittest.TestCase):
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils/",
             ["*.foo", "*.bar"],
@@ -186,7 +201,7 @@ class S3UtilsTest(unittest.TestCase):
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils/",
             [],
@@ -200,7 +215,7 @@ class S3UtilsTest(unittest.TestCase):
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils",
             [],
@@ -220,11 +235,11 @@ class S3UtilsTest(unittest.TestCase):
     @ddt.idata(data_s3_listfile)
     @ddt.unpack
     def test_s3_listfile(self, src, bucket, prefix, include_patterns, exclude_patterns, depth, expect):
-        with moto.mock_aws(), s3_make_client(region_name="us-east-1") as client:
+        with moto.mock_aws(), s3_make_client() as client:
             client.client.create_bucket(Bucket=bucket)
 
             s3_sync_upload(client,
-                           src.format(res_dir=resources_directory),
+                           os.path.join(resources_directory, src),
                            bucket,
                            prefix)
 
@@ -240,137 +255,137 @@ class S3UtilsTest(unittest.TestCase):
 
     data_s3_sync = [
         (
-            "{res_dir}/unittest/s3utils/",
-            "{tmp_dir}/unittest/s3utils/",
+            "unittest/s3utils/",
+            "unittest/s3utils/",
             "dummy-bucket",
             "unittest/s3utils/",
             [],
             [],
             0,
             [
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.bar",
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.baz",
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.bar.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.foo",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.foo.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.bar.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/dir.foo.bar.baz/file.foo.bar.baz",
+                "unittest/s3utils/dir.baz/file.foo.bar",
+                "unittest/s3utils/dir.baz/file.foo.baz",
+                "unittest/s3utils/dir.baz/file.bar.baz",
+                "unittest/s3utils/dir.foo/file.bar",
+                "unittest/s3utils/dir.foo/file.baz",
+                "unittest/s3utils/dir.foo/file.foo",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.foo.baz",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.bar.baz",
+                "unittest/s3utils/dir.foo/dir.foo.bar/dir.foo.bar.baz/file.foo.bar.baz",
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
-            "{tmp_dir}/unittest/s3utils",
+            "unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils/",
             [],
             [],
             0,
             [
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.bar",
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.baz",
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.bar.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.foo",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.foo.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.bar.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/dir.foo.bar.baz/file.foo.bar.baz",
+                "unittest/s3utils/dir.baz/file.foo.bar",
+                "unittest/s3utils/dir.baz/file.foo.baz",
+                "unittest/s3utils/dir.baz/file.bar.baz",
+                "unittest/s3utils/dir.foo/file.bar",
+                "unittest/s3utils/dir.foo/file.baz",
+                "unittest/s3utils/dir.foo/file.foo",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.foo.baz",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.bar.baz",
+                "unittest/s3utils/dir.foo/dir.foo.bar/dir.foo.bar.baz/file.foo.bar.baz",
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils/dir.foo",
-            "{tmp_dir}/unittest/s3utils/dir.foo",
+            "unittest/s3utils/dir.foo",
+            "unittest/s3utils/dir.foo",
             "dummy-bucket",
             "unittest/s3utils/dir.foo",
             [],
             [],
             0,
             [
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.foo",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.foo.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.bar.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/dir.foo.bar.baz/file.foo.bar.baz",
+                "unittest/s3utils/dir.foo/file.bar",
+                "unittest/s3utils/dir.foo/file.baz",
+                "unittest/s3utils/dir.foo/file.foo",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.foo.baz",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.bar.baz",
+                "unittest/s3utils/dir.foo/dir.foo.bar/dir.foo.bar.baz/file.foo.bar.baz",
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils/dir.baz",
-            "{tmp_dir}/unittest/s3utils/dir.baz",
+            "unittest/s3utils/dir.baz",
+            "unittest/s3utils/dir.baz",
             "dummy-bucket",
             "unittest/s3utils/dir.baz",
             [],
             [],
             0,
             [
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.bar",
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.baz",
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.bar.baz",
+                "unittest/s3utils/dir.baz/file.foo.bar",
+                "unittest/s3utils/dir.baz/file.foo.baz",
+                "unittest/s3utils/dir.baz/file.bar.baz",
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
-            "{tmp_dir}/unittest/s3utils",
+            "unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils/",
             ["*.foo", "*.bar"],
             ["*.foo.bar"],
             0,
             [
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.foo",
+                "unittest/s3utils/dir.foo/file.bar",
+                "unittest/s3utils/dir.foo/file.foo",
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
-            "{tmp_dir}/unittest/s3utils",
+            "unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils/",
             ["*.foo", "*.bar"],
             [],
             0,
             [
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.foo",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
+                "unittest/s3utils/dir.baz/file.foo.bar",
+                "unittest/s3utils/dir.foo/file.bar",
+                "unittest/s3utils/dir.foo/file.foo",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
-            "{tmp_dir}/unittest/s3utils",
+            "unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils/",
             [],
             ["*.baz"],
             0,
             [
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.foo",
-                "{tmp_dir}/unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
+                "unittest/s3utils/dir.baz/file.foo.bar",
+                "unittest/s3utils/dir.foo/file.bar",
+                "unittest/s3utils/dir.foo/file.foo",
+                "unittest/s3utils/dir.foo/dir.foo.bar/file.foo.bar",
             ],
         ),
         (
-            "{res_dir}/unittest/s3utils",
-            "{tmp_dir}/unittest/s3utils",
+            "unittest/s3utils",
+            "unittest/s3utils",
             "dummy-bucket",
             "unittest/s3utils",
             [],
             [],
             2,
             [
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.bar",
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.foo.baz",
-                "{tmp_dir}/unittest/s3utils/dir.baz/file.bar.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.bar",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.baz",
-                "{tmp_dir}/unittest/s3utils/dir.foo/file.foo",
+                "unittest/s3utils/dir.baz/file.foo.bar",
+                "unittest/s3utils/dir.baz/file.foo.baz",
+                "unittest/s3utils/dir.baz/file.bar.baz",
+                "unittest/s3utils/dir.foo/file.bar",
+                "unittest/s3utils/dir.foo/file.baz",
+                "unittest/s3utils/dir.foo/file.foo",
             ],
         ),
     ]
@@ -378,11 +393,11 @@ class S3UtilsTest(unittest.TestCase):
     @ddt.idata(data_s3_sync)
     @ddt.unpack
     def test_s3_sync(self, src, dst, bucket, prefix, include_patterns, exclude_patterns, depth, expect):
-        with moto.mock_aws(), s3_make_client(region_name="us-east-1") as client:
+        with moto.mock_aws(), s3_make_client() as client:
             client.client.create_bucket(Bucket=bucket)
 
             s3_sync_upload(client,
-                           src.format(res_dir=resources_directory),
+                           os.path.join(resources_directory, src),
                            bucket,
                            prefix,
                            include_patterns=include_patterns,
@@ -393,16 +408,16 @@ class S3UtilsTest(unittest.TestCase):
                 s3_sync_download(client,
                                  bucket,
                                  prefix,
-                                 dst.format(tmp_dir=temp_directory))
+                                 os.path.join(temp_directory, dst))
 
-                self.assertSetEqual(set(map(lambda x: norm_path(x), listfile(dst.format(tmp_dir=temp_directory)))),
-                                    set(map(lambda x: norm_path(x.format(tmp_dir=temp_directory)), expect)))
+                self.assertSetEqual(set(map(lambda x: norm_path(x), listfile(os.path.join(temp_directory, dst)))),
+                                    set(map(lambda x: norm_path(os.path.join(temp_directory, x)), expect)))
 
-        with moto.mock_aws(), s3_make_client(region_name="us-east-1") as client:
+        with moto.mock_aws(), s3_make_client() as client:
             client.client.create_bucket(Bucket=bucket)
 
             s3_sync_upload(client,
-                           src.format(res_dir=resources_directory),
+                           os.path.join(resources_directory, src),
                            bucket,
                            prefix)
 
@@ -410,19 +425,19 @@ class S3UtilsTest(unittest.TestCase):
                 s3_sync_download(client,
                                  bucket,
                                  prefix,
-                                 dst.format(tmp_dir=temp_directory),
+                                 os.path.join(temp_directory, dst),
                                  include_patterns=include_patterns,
                                  exclude_patterns=exclude_patterns,
                                  depth=depth)
 
-                self.assertSetEqual(set(map(lambda x: norm_path(x), listfile(dst.format(tmp_dir=temp_directory)))),
-                                    set(map(lambda x: norm_path(x.format(tmp_dir=temp_directory)), expect)))
+                self.assertSetEqual(set(map(lambda x: norm_path(x), listfile(os.path.join(temp_directory, dst)))),
+                                    set(map(lambda x: norm_path(os.path.join(temp_directory, x)), expect)))
 
-        with moto.mock_aws(), s3_make_client(region_name="us-east-1") as client:
+        with moto.mock_aws(), s3_make_client() as client:
             client.client.create_bucket(Bucket=bucket)
 
             s3_sync_upload(client,
-                           src.format(res_dir=resources_directory),
+                           os.path.join(resources_directory, src),
                            bucket,
                            prefix,
                            include_patterns=include_patterns,
@@ -432,12 +447,12 @@ class S3UtilsTest(unittest.TestCase):
                 s3_sync_download(client,
                                  bucket,
                                  prefix,
-                                 dst.format(tmp_dir=temp_directory),
+                                 os.path.join(temp_directory, dst),
                                  exclude_patterns=exclude_patterns,
                                  depth=depth)
 
-                self.assertSetEqual(set(map(lambda x: norm_path(x), listfile(dst.format(tmp_dir=temp_directory)))),
-                                    set(map(lambda x: norm_path(x.format(tmp_dir=temp_directory)), expect)))
+                self.assertSetEqual(set(map(lambda x: norm_path(x), listfile(os.path.join(temp_directory, dst)))),
+                                    set(map(lambda x: norm_path(os.path.join(temp_directory, x)), expect)))
 
     data_s3_text = [
         ("dummy-bucket", "dummy/key", "dummy content", None),
@@ -451,8 +466,186 @@ class S3UtilsTest(unittest.TestCase):
     @ddt.idata(data_s3_text)
     @ddt.unpack
     def test_s3_text(self, bucket, key, text, encoding):
-        with moto.mock_aws(), s3_make_client(region_name="us-east-1") as client:
+        with moto.mock_aws(), s3_make_client() as client:
             client.client.create_bucket(Bucket=bucket)
 
             s3_push_text(client, text, bucket, key, encoding=encoding)
             self.assertEqual(s3_pull_text(client, bucket, key, encoding=encoding), text)
+
+    def test_s3_estimate_archive_ranged_requests(self):
+        with (
+            moto.mock_aws(),
+            moto_server_endpoint_url() as endpoint_url,
+            s3_make_client(endpoint_url=endpoint_url) as client,
+        ):
+            client.client.create_bucket(Bucket="dummy-bucket")
+
+            s3_sync_upload(client,
+                           os.path.join(resources_directory, "unittest", "s3utils_archive"),
+                           "dummy-bucket",
+                           "s3utils_archive")
+
+            local_root = os.path.join(resources_directory, "unittest", "s3utils")
+
+            members = [os.path.relpath(file_path, local_root) for file_path in listfile(local_root)]
+
+            ranged_total_size, archive_size, missing_members = s3_estimate_archive_ranged_requests(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
+                members,
+                member_header_overhead=0,
+                central_directory_overhead=0,
+            )
+
+            self.assertEqual(archive_size - ranged_total_size, 2470)
+            self.assertEqual(len(missing_members), 0)
+
+            ranged_total_size, archive_size, missing_members = s3_estimate_archive_ranged_requests(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
+                members,
+                member_header_overhead=0,
+                central_directory_overhead=0,
+            )
+
+            self.assertEqual(archive_size - ranged_total_size, 2470)
+            self.assertEqual(len(missing_members), 0)
+
+    def test_s3_open_archive_member(self):
+        with (
+            moto.mock_aws(),
+            moto_server_endpoint_url() as endpoint_url,
+            s3_make_client(endpoint_url=endpoint_url) as client,
+        ):
+            client.client.create_bucket(Bucket="dummy-bucket")
+
+            s3_sync_upload(client,
+                           os.path.join(resources_directory, "unittest", "s3utils_archive"),
+                           "dummy-bucket",
+                           "s3utils_archive")
+
+            local_root = os.path.join(resources_directory, "unittest", "s3utils")
+
+            members = [os.path.relpath(file_path, local_root) for file_path in listfile(local_root)]
+
+            for member in members:
+                with (
+                    open(os.path.join(local_root, member), "r") as local_fh,
+                    s3_open_archive_member(
+                        client,
+                        "s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
+                        member,
+                    ) as s3_fh,
+                ):
+                    self.assertEqual(local_fh.read(), s3_fh.read())
+
+                with (
+                    open(os.path.join(local_root, member), "r") as local_fh,
+                    s3_open_archive_member(
+                        client,
+                        "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
+                        member,
+                    ) as s3_fh,
+                ):
+                    self.assertEqual(local_fh.read(), s3_fh.read())
+
+    def test_s3_open_archive_members(self):
+        with (
+            moto.mock_aws(),
+            moto_server_endpoint_url() as endpoint_url,
+            s3_make_client(endpoint_url=endpoint_url) as client,
+        ):
+            client.client.create_bucket(Bucket="dummy-bucket")
+
+            s3_sync_upload(client,
+                           os.path.join(resources_directory, "unittest", "s3utils_archive"),
+                           "dummy-bucket",
+                           "s3utils_archive")
+
+            local_root = os.path.join(resources_directory, "unittest", "s3utils")
+
+            members = [os.path.relpath(file_path, local_root) for file_path in listfile(local_root)]
+
+            for member, opener in s3_open_archive_members(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
+                members,
+                threshold=1.0,
+                member_header_overhead=0,
+                central_directory_overhead=0,
+            ):
+                with open(os.path.join(local_root, member), "r") as local_fh, opener() as s3_fh:
+                    self.assertEqual(local_fh.read(), s3_fh.read())
+
+            for member, opener in s3_open_archive_members(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
+                members,
+                threshold=1.0,
+                member_header_overhead=0,
+                central_directory_overhead=0,
+            ):
+                with open(os.path.join(local_root, member), "r") as local_fh, opener() as s3_fh:
+                    self.assertEqual(local_fh.read(), s3_fh.read())
+
+            for member, opener in s3_open_archive_members(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
+                members,
+                threshold=0.0,
+            ):
+                with open(os.path.join(local_root, member), "r") as local_fh, opener() as s3_fh:
+                    self.assertEqual(local_fh.read(), s3_fh.read())
+
+            for member, opener in s3_open_archive_members(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
+                members,
+                threshold=0.0,
+            ):
+                with open(os.path.join(local_root, member), "r") as local_fh, opener() as s3_fh:
+                    self.assertEqual(local_fh.read(), s3_fh.read())
+
+            for member, opener in s3_open_archive_members(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
+                members,
+                "rb",
+                threshold=1.0,
+                member_header_overhead=0,
+                central_directory_overhead=0,
+            ):
+                with open(os.path.join(local_root, member), "rb") as local_fh, opener() as s3_fh:
+                    self.assertEqual(local_fh.read(), s3_fh.read())
+
+            for member, opener in s3_open_archive_members(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
+                members,
+                "rb",
+                threshold=1.0,
+                member_header_overhead=0,
+                central_directory_overhead=0,
+            ):
+                with open(os.path.join(local_root, member), "rb") as local_fh, opener() as s3_fh:
+                    self.assertEqual(local_fh.read(), s3_fh.read())
+
+            for member, opener in s3_open_archive_members(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
+                members,
+                "rb",
+                threshold=0.0,
+            ):
+                with open(os.path.join(local_root, member), "rb") as local_fh, opener() as s3_fh:
+                    self.assertEqual(local_fh.read(), s3_fh.read())
+
+            for member, opener in s3_open_archive_members(
+                client,
+                "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
+                members,
+                "rb",
+                threshold=0.0,
+            ):
+                with open(os.path.join(local_root, member), "rb") as local_fh, opener() as s3_fh:
+                    self.assertEqual(local_fh.read(), s3_fh.read())
